package cn.com.qmth.examcloud.core.reports.service.impl;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.persistence.criteria.Predicate;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.api.commons.exchange.PageInfo;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.reports.api.request.SaveExamDataReq;
import cn.com.qmth.examcloud.core.reports.base.bean.ExamDataBean;
import cn.com.qmth.examcloud.core.reports.dao.ExamDataRepo;
import cn.com.qmth.examcloud.core.reports.dao.ExamStudentCountRepo;
import cn.com.qmth.examcloud.core.reports.dao.entity.ExamDataEntity;
import cn.com.qmth.examcloud.core.reports.dao.entity.ExamStudentCountEntity;
import cn.com.qmth.examcloud.core.reports.service.ExamDataService;

@Service
public class ExamDataServiceImpl implements ExamDataService {

    @Autowired
    private ExamDataRepo examDataRepo;

    @Autowired
    private ExamStudentCountRepo examStudentCountRepo;

    @Override
    public PageInfo<ExamDataBean> queryPage(Long rootOrgId, String examName, Integer pageNo, Integer pageSize) {
        Specification<ExamDataEntity> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            if (null != rootOrgId) {
                predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));
            }
            if (StringUtils.isNotBlank(examName)) {
                predicates.add(cb.like(root.get("examName"), toSqlSearchPattern(examName)));
            }
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };

        List<ExamDataEntity> es = examDataRepo.findAll(specification);

        List<ExamDataBean> ret = Lists.newArrayList();

        for (ExamDataEntity cur : es) {
            ExamDataBean bean = new ExamDataBean();
            bean.setPlanCount(cur.getPlanCount());
            bean.setCompleteCount(cur.getCompleteCount());
            bean.setCompleteRatio(getPercentage(bean.getCompleteCount(), bean.getPlanCount()));
            bean.setExamEndDate(cur.getEndTime());
            bean.setExamStartDate(cur.getStartTime());
            bean.setExamId(cur.getExamId());
            bean.setExamName(cur.getExamName());
            bean.setExamType(cur.getExamType());
            bean.setRootOrgId(cur.getRootOrgId());
            bean.setOnlineCount(0);
            ret.add(bean);
        }
        fillOnlineCount(ret, rootOrgId);
        Collections.sort(ret, new Comparator<ExamDataBean>() {

            @Override
            public int compare(ExamDataBean o1, ExamDataBean o2) {
                if (o1.getOnlineCount() > o2.getOnlineCount()) {
                    return -1;
                } else if (o1.getOnlineCount() < o2.getOnlineCount()) {
                    return 1;
                } else {
                    return 0;
                }
            }

        });
        int total = ret.size();
        ret = ret.stream().skip((pageNo - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
        PageInfo<ExamDataBean> pi = new PageInfo<ExamDataBean>();
        pi.setLimit(pageSize);
        pi.setList(ret);
        pi.setPages(pageNo);
        pi.setTotal(total);
        return pi;
    }

    private Double getPercentage(Integer a, Integer b) {
        if (a == null) {
            a = 0;
        }
        if (b == null) {
            return null;
        }
        if (b == 0) {
            return null;
        }
        Double da = Double.valueOf(a);
        Double db = Double.valueOf(b);
        BigDecimal bd = new BigDecimal(da * 100 / db);
        Double tem = bd.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
        return tem;
    }

    private void fillOnlineCount(List<ExamDataBean> ret, Long rootOrgId) {
        if (CollectionUtils.isEmpty(ret)) {
            return;
        }
        List<Long> temList = ret.stream().map(str -> str.getExamId()).collect(Collectors.toList());
        List<ExamStudentCountEntity> sl = examStudentCountRepo.getByExamIdsRootOrgId(temList, rootOrgId);
        if (CollectionUtils.isEmpty(sl)) {
            return;
        }
        Map<Long, Integer> map = sl.stream().collect(Collectors.toMap(ExamStudentCountEntity::getExamId,
                account -> account.getOnlineCount(), (key1, key2) -> key2));
        for (ExamDataBean b : ret) {
            if (map.get(b.getExamId()) != null) {
                b.setOnlineCount(map.get(b.getExamId()));
            }
        }
    }

    @Transactional
    @Override
    public void save(SaveExamDataReq req) {
        Long rootOrgId = req.getRootOrgId();
        Integer planCount = req.getPlanCount();
        Integer completeCount = req.getCompleteCount();
        Date endTime = req.getEndTime();
        Date startTime = req.getStartTime();
        Long examId = req.getExamId();
        String examName = req.getExamName();
        String examType = req.getExamType();
        if (planCount == null) {
            planCount = 0;
        }
        if (completeCount == null) {
            completeCount = 0;
        }
        if (rootOrgId == null) {
            throw new StatusException("1000001", "rootOrgId不能为空");
        }
        if (examId == null) {
            throw new StatusException("1000002", "examId不能为空");
        }
        if (examName == null) {
            throw new StatusException("1000003", "examName不能为空");
        }
        if (examType == null) {
            throw new StatusException("1000004", "examType不能为空");
        }
        ExamDataEntity ed = examDataRepo.findByRootOrgIdAndExamId(rootOrgId, examId);
        if (ed == null) {
            Date now = new Date();
            ed = new ExamDataEntity();
            ed.setEndTime(endTime);
            ed.setStartTime(startTime);
            ed.setExamId(examId);
            ed.setExamName(examName);
            ed.setExamType(examType);
            ed.setRootOrgId(rootOrgId);
            ed.setCreationTime(now);
            ed.setUpdateTime(now);
        }
        ed.setPlanCount(planCount.intValue());
        ed.setCompleteCount(completeCount.intValue());
        examDataRepo.save(ed);
    }

    private String toSqlSearchPattern(String column) {
        if (StringUtils.isBlank(column)) {
            return "%";
        } else {
            column = column.trim().replaceAll("\\s", "%");
            column = "%" + column + "%";
            return column;
        }
    }
}
